/** 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version. 
 *  
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details. 
 *  
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
 * 
 * @version 2.0
 * DisChoco
 * Graph.java
 */ 
package gui.visualizing;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

/**
 * 
 * 
 * @authors 
 * <a href="mailto:ezzahir@ensa-agadir.ac.ma?subject=DisChoco &cc=wahbi@lirmm.fr &cc=bessiere@lirmm.fr">
 * Redoaune Ezzahir</a> and 
 * <a href="mailto:wahbi@lirmm.fr?subject=DisChoco &cc=ezzahir@ensa-agadir.ac.ma &cc=bessiere@lirmm.fr">
 * Mohamed Wahbi</a>
 * 
 */

public class Graph<V, E> {

	/** Map of vertices to Pair of adjacency maps {incoming, outgoing} of neighboring vertices to incident edges */
	Map<V, Map<V, E>> vertices;
	/** Map of edges to incident vertex pairs */
	Map<E, Pair<V>> edges;

	/**
	 * Creates an instance.
	 */
	public Graph() {
		vertices = new HashMap<V, Map<V, E>>();
		edges = new HashMap<E, Pair<V>>();
	}

	public boolean addEdge(E e, Pair<V> endpoints) {
		if (endpoints == null) return false;
		V v1 = endpoints.getFirst();
		V v2 = endpoints.getSecond();
		if (findEdge(v1, v2) != null) return false;
		edges.put(e, endpoints);
		if (!vertices.containsKey(v1)) this.addVertex(v1);
		if (!vertices.containsKey(v2)) this.addVertex(v2);
		vertices.get(v1).put(v2, e);
		vertices.get(v2).put(v1, e);
		return true;
	}

	public boolean addVertex(V v) {
		if (v == null) { throw new IllegalArgumentException("vertex may not be null"); }
		if (!containsVertex(v)) {
			vertices.put(v, new HashMap<V, E>());
			return true;
		} else {
			return false;
		}
	}

	public boolean removeVertex(V v) {
		if (!containsVertex(v)) return false;
		for (E e : new ArrayList<E>(vertices.get(v).values()))
			removeEdge(e);
		vertices.remove(v);
		return true;
	}

	public boolean removeEdge(E e) {
		if (!containsEdge(e)) return false;
		Pair<V> endpoints = getEndpoints(e);
		V v1 = endpoints.getFirst();
		V v2 = endpoints.getSecond();
		vertices.get(v1).remove(v2);
		vertices.get(v2).remove(v1);
		edges.remove(e);
		return true;
	}

	public E findEdge(V v1, V v2) {
		if (!containsVertex(v1) || !containsVertex(v2)) return null;
		return vertices.get(v1).get(v2);
	}

	public Collection<E> findEdgeSet(V v1, V v2) {
		if (!containsVertex(v1) || !containsVertex(v2)) return null;
		ArrayList<E> edge_collection = new ArrayList<E>(1);
		E e = findEdge(v1, v2);
		if (e == null) return edge_collection;
		edge_collection.add(e);
		return edge_collection;
	}

	public Pair<V> getEndpoints(E e) {
		return edges.get(e);
	}

	public Collection<E> getEdges() {
		return Collections.unmodifiableCollection(edges.keySet());
	}

	public Collection<V> getVertices() {
		return Collections.unmodifiableCollection(vertices.keySet());
	}

	public boolean containsVertex(V v) {
		return vertices.containsKey(v);
	}

	public boolean containsEdge(E e) {
		return edges.containsKey(e);
	}

	public int getEdgeCount() {
		return edges.size();
	}

	public int getVertexCount() {
		return vertices.size();
	}

	public Collection<V> getNeighbors(V v) {
		if (!containsVertex(v)) return null;
		return Collections.unmodifiableCollection(vertices.get(v).keySet());
	}

	public Collection<E> getEdges(V v) {
		if (!containsVertex(v)) return null;
		return Collections.unmodifiableCollection(vertices.get(v).values());
	}
}

